/*******************************************************************************
 * Copyright (c) 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     Erich Gamma (erich_gamma@ch.ibm.com) and
 * 	   Kent Beck (kent@threeriversinstitute.org)
 *******************************************************************************/
package org.eclipse.contribution.junit;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.util.SafeRunnable;

public class JUnitPlugin extends Plugin {
	private static final String listenerId= "org.eclipse.contribution.junit.listeners";

	private static JUnitPlugin instance;
	private List listeners;

	public JUnitPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		instance= this;
	}

	public static JUnitPlugin getPlugin() {
		return instance;
	}

	private List getListeners() {
		if (listeners == null)
			listeners= computeListeners();
		return listeners;
	}

	private List computeListeners() {
		IPluginRegistry registry= Platform.getPluginRegistry();
		IExtensionPoint extensionPoint= registry.getExtensionPoint(listenerId);
		IExtension[] extensions= extensionPoint.getExtensions();
		ArrayList results= new ArrayList();
		for (int i= 0; i < extensions.length; i++) {
			IConfigurationElement[] elements= extensions[i].getConfigurationElements();
			for (int j= 0; j < elements.length; j++) {
				try {
					Object listener= elements[j].createExecutableExtension("class");
					if (listener instanceof ITestRunListener)
						results.add(listener);
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}
		return results;
	}
	
	public void addTestListener(ITestRunListener listener) {
		getListeners().add(listener);
	}

	public void removeTestListener(ITestRunListener listener) {
		getListeners().remove(listener);
	}

	public void fireTestsStarted(final int count) { // Only public for testing purposes
		for (Iterator all= getListeners().iterator(); all.hasNext();) {
			final ITestRunListener each= (ITestRunListener) all.next();
			Platform.run(new SafeRunnable() {
				public void run() throws Exception {
					each.testsStarted(count);
				}
			});
		}
	}

	public void fireTestsFinished() {
		for (Iterator all= getListeners().iterator(); all.hasNext();) {
			final ITestRunListener each= (ITestRunListener) all.next();
			Platform.run(new SafeRunnable() {
				public void run() throws Exception {
					each.testsFinished();
				}
			});
		}
	}

	public void fireTestStarted(final String klass, final String methodName) {
		for (Iterator all= getListeners().iterator(); all.hasNext();) {
			final ITestRunListener each= (ITestRunListener) all.next();
			Platform.run(new SafeRunnable() {
				public void run() throws Exception {
					each.testStarted(klass, methodName);
				}
			});
		}
	}

	public void fireTestFailed(final String klass, final String method, final String trace) {
		for (Iterator all= getListeners().iterator(); all.hasNext();) {
			final ITestRunListener each= (ITestRunListener) all.next();
			Platform.run(new SafeRunnable() {
				public void run() throws Exception {
					each.testFailed(klass, method, trace);
				}
			});
		}
	}

	public void run(IType type) throws CoreException {
		new TestRunner().run(type);
	}
}
